#!/usr/bin/env bash

# HSC Analysis Pipeline
# Copyright (C) 2015-2017  HSC Data Analysis Software Team
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


# Description
# create-rootdir.sh ROOTDIR [REFDIR]
#
# This script prepares a root directory for data analysis. It is not "/" but
# is a directory where input FITS files and output products will be saved.
#
# Usually, you should call this script with two arguments: ROOTDIR and REFDIR.
# ROOTDIR is the root directory for data analysis explained above. REFDIR
# is a directory that contains a reference catalog for photometric and
# astrometric calibration. "REFDIR/config.py" must exist because this script
# checks for it to confirm you have given a correct path.
#
# If REFDIR is not specified, no reference catalog will be set in ROOTDIR.
# To set one afterwards, run this script again with a valid REFDIR.
# End Description

startup()
(
  help=false

  if [[ $# -eq 0 ]]
  then
    help=true
  else
    for i in "$@"
    do
      if  [[ "$i" = "-h" ]] ||
          [[ "$i" = "--h" ]] ||
          [[ "$i" = "--he" ]] ||
          [[ "$i" = "--hel" ]] ||
          [[ "$i" = "--help" ]]
      then
        help=true
        break
      fi
    done
  fi

  if $help
  then
    cat "$0" | sed -n '/^# Description/,/^# End Description/ { s/^\(# \|#\)// ; p }' |
      tail -n+2 | head -n-1
    exit 0
  fi

  if [[ $# -ge 3 ]]
  then
    echo "Too many arguments. See $(basename "$0") --help."
    exit 1
  fi

  main "$1" "$2"
)

main()
(
  set -e

  rootdir="$1"
  refdir="$2"

  if [[ -z "${rootdir}" ]]
  then
    echo "Specify the root directory for analysis."
    exit 1
  fi

  if  [[ -n "${refdir}" ]] &&
      [[ ! -e "${refdir}"/config.py ]]
  then
    echo "The directory of reference catalogs you specified does not seem to be correct."
    exit 1
  fi

  if [[ -z "$OBS_SUBARU_DIR" ]]
  then
    dirs=("$(dirname "$0")"/*/*/obs_subaru/*/hsc)
    if  [[ ${#dirs[@]} -ne 1 ]] ||
        [[ ! -d "${dirs[0]}" ]]
    then
      echo '$OBS_SUBARU_DIR is not set and we failed in guessing it.'
      echo 'Set up the pipeline first, and run this script again.'
      exit 1
    fi

    pushd "$(dirname "${dirs[0]}")" > /dev/null
    OBS_SUBARU_DIR="$PWD"
    popd > /dev/null
  fi

  mkdir -p "${rootdir}"

  if [[ ! -e "${rootdir}/_mapper" ]]
  then
    echo 'lsst.obs.hsc.HscMapper' > "${rootdir}/_mapper"
  fi

  if [[ ! -e "${rootdir}"/CALIB/BFKERNEL/brighter_fatter_kernel.pkl ]]
  then
    mkdir -p "${rootdir}"/CALIB/BFKERNEL
    ln -s "${OBS_SUBARU_DIR}"/hsc/brighter_fatter_kernel.pkl "${rootdir}"/CALIB/BFKERNEL/
  fi

  if  [[ -n "${refdir}" ]] &&
      [[ ! -e "${rootdir}"/ref_cats/"$(basename "${refdir}")" ]]
  then
    pushd ${refdir} > /dev/null
    refdir="$PWD"
    popd > /dev/null

    mkdir -p "${rootdir}"/ref_cats
    ln -s "${refdir}" "${rootdir}"/ref_cats/
  fi
)

startup "$@"
